﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;
using System.Windows.Input;
using Windows.Storage;
using VisualStateDemo.Infra;
using VisualStateDemo.State;
using VisualStateDemo.State.VisualStates;
using VisualStateDemo.ViewModels.Items;

namespace VisualStateDemo.ViewModels
{
    public class MainPageVm : ViewModelBase
    {
        private List<DayTemperature> _yearData; 

        public StateHandler StateHandler { get; private set; }
        public DayTemperature SimulationData { get; private set; }
        public ICommand ContinueToStartCommand { get; private set; }
        public ICommand StartSimulationCommand { get; private set; }
        public ICommand CelsiusCommand { get; private set; }
        public ICommand FahrenheitCommand { get; private set; }


        public MainPageVm()
        {
            if (DesignMode)
            {
                StateHandler = new StateHandler();
                ShowSampleData();
            }
            else
            {
                StateHandler = new StateHandler();
                StateHandler.SetScaleState(ScaleStates.Celsius);
                StateHandler.SetScreenState(ScreenStates.Welcome);

                ContinueToStartCommand = new DelegateCommand(OnContinueToStartCommand);
                StartSimulationCommand = new DelegateCommand(OnStartSimulationCommand);
                CelsiusCommand = new DelegateCommand(OnCelsiusCommand);
                FahrenheitCommand = new DelegateCommand(OnFahrenheitCommand);
            }
        }

        private void OnFahrenheitCommand()
        {
           if(StateHandler.ScaleState!=ScaleStates.Fahrenheit) 
               StateHandler.SetScaleState(ScaleStates.Fahrenheit);
        }

        
        private void OnCelsiusCommand()
        {
            if (StateHandler.ScaleState != ScaleStates.Celsius) 
            StateHandler.SetScaleState(ScaleStates.Celsius);
        }

        private async void OnContinueToStartCommand()
        {
            StateHandler.SetScreenState(ScreenStates.Start);
            var data = await GetSimulationDataAsync();
            _yearData = data;
        }

        private async void OnStartSimulationCommand()
        {
            StateHandler.SetScreenState(ScreenStates.Simulation);

            foreach (var dayTemperature in _yearData)
            {
                if (StateHandler.ScaleState == ScaleStates.Fahrenheit)
                    dayTemperature.Temperature = ConvertToFahrenheit(dayTemperature.Temperature);

                SimulationData = dayTemperature;

                var season = GetSeason(dayTemperature.Date);
                if (StateHandler.SeasonState!=season)
                    StateHandler.SetSeasonState(season);

                await Task.Delay(50);
            }
            StateHandler.SetScreenState(ScreenStates.Done);
        }

        public static decimal ConvertToFahrenheit(decimal celsius)
        {
            return celsius * 9 / 5 + 32;
        }


        private SeasonStates GetSeason(DateTime date)
        {
            switch (date.Month)
            {
                case 1:
                case 2:
                case 3:
                case 11:
                case 12:
                    return SeasonStates.Winter;
                case 4:
                case 5:
                    return SeasonStates.Spring;
                case 6:
                case 7:
                case 8:
                    return SeasonStates.Summer;
                case 9:
                case 10:
                    return SeasonStates.Autumn;
                default:
                    return SeasonStates.Unknown;
            }
        }


        private void ShowSampleData()
        {
            SimulationData= new DayTemperature {Date = DateTime.Now, Temperature = -1};
        }

        private static async Task<List<DayTemperature>> GetSimulationDataAsync()
        {
            var file = await StorageFile.GetFileFromApplicationUriAsync(new Uri("ms-appx:///Data/2013malmo.csv"));
            var stream = await file.OpenStreamForReadAsync();
            var data = new List<DayTemperature>();
            using (var sr = new StreamReader(stream))
            {
                while (sr.Peek() >= 0)
                {
                    var readLine = sr.ReadLine();
                    if (readLine == null) continue;
                    var day = readLine.Split(Convert.ToChar(";"));
                    data.Add(new DayTemperature { Date = Convert.ToDateTime(day[0]), Temperature = Convert.ToDecimal(day[1]) });
                }
            }
            return data;
        }

    }
}
